@@ -37,10 +37,12 @@ extern char tbuf[1024];			/* Temp buffer for anybody. */
 extern int entries;			/* Number of people. */
 extern DB *db;				/* Database. */
 
-#if NETSEC
+#if NETPROC
+#include <net/netproc.h>
+
 extern void *request;
 extern int requestlen;
-#endif /* NETSEC */
+#endif /* NETPROC */
 
 void	 enter_lastlog __P((PERSON *));
 PERSON	*enter_person __P((struct passwd *));
