@@ -43,10 +43,14 @@ static char sccsid[] = "@(#)lprint.c	8.3 (Berkeley) 4/28/95";
 #include <sys/time.h>
 #include <fcntl.h>
 #include <time.h>
-#ifndef linux
+#if !__linux__ && !__FreeBSD__
 #include <tzfile.h>
-#endif /* linux */
+#endif /* !__linux__ && !__FreeBSD__ */
+#if __linux__
+#include <db_185.h>
+#else /* __linux__ */
 #include <db.h>
+#endif /* __linux__ */
 #include <err.h>
 #include <pwd.h>
 #include <utmp.h>
@@ -214,23 +218,23 @@ lprint(pn)
 			}
 			tp = localtime(&w->loginat);
 			t = asctime(tp);
-#ifdef linux
+#if __linux__
 			if (now - w->loginat > 1576800)
 				cpr = printf("Last login %.16s %.4s (%s) on %s",
 				    t, t + 20, *tzname, w->tty);
 			else
 				cpr = printf("Last login %.16s (%s) on %s",
 				    t, *tzname, w->tty);
-#else /* linux */
+#else /* __linux__ */
 			tzn = tp->tm_zone;
-			if (now - w->loginat > SECSPERDAY * DAYSPERNYEAR / 2)
+			if (now - w->loginat > 1576800)
 				cpr =
 				    printf("Last login %.16s %.4s (%s) on %s",
 				    t, t + 20, tzn, w->tty);
 			else
 				cpr = printf("Last login %.16s (%s) on %s",
 				    t, tzn, w->tty);
-#endif /* linux */
+#endif /* __linux__ */
 			break;
 		}
 		if (*w->host) {
