@@ -1,6 +1,6 @@
 /*
 %%% portions-copyright-cmetz-96
-Portions of this software are Copyright 1996-1997 by Craig Metz, All Rights
+Portions of this software are Copyright 1996-1998 by Craig Metz, All Rights
 Reserved. The Inner Net License Version 2 applies to these portions of
 the software.
 You should have received a copy of the license with this software. If
@@ -99,11 +99,11 @@ netfinger(name)
 	  return;
 	}
 
-#if NETSEC
+#if NETPROC
         s = inner_connect(ai, request, requestlen, NULL, NULL, "finger", &ai2);
-#else /* NETSEC */
+#else /* NETPROC */
         s = inner_connect(ai, NULL, 0, NULL, NULL, "finger", &ai2);
-#endif /* NETSEC */
+#endif /* NETPROC */
 	if (s >= 0)
           printf("[%s]\n", ai2->ai_canonname);
 	freeaddrinfo(ai);
