@@ -1274,7 +1274,11 @@ shell(argc, argv)
 	pid_t pid;
 	sig_t old1, old2;
 	char shellnam[40], *shell, *namep; 
+#if 0
 	union wait status;
+#else /* 0 */
+	int status;
+#endif /* 0 */
 
 	old1 = signal (SIGINT, SIG_IGN);
 	old2 = signal (SIGQUIT, SIG_IGN);
@@ -2030,8 +2034,16 @@ setpassive(argc, argv)
 	int argc;
 	char *argv[];
 {
-
+#if INET6
+  if (passivemode > 1) {
+    fprintf(stderr, "Passive mode is required on this connection; restart to change.\n");
+    return;
+  };
+    
+  passivemode = passivemode ? 0 : 1;
+#else /* INET6 */
 	passivemode = !passivemode;
+#endif /* INET6 */
 	printf("Passive mode %s.\n", onoff(passivemode));
 	code = passivemode;
 }
