@@ -1,6 +1,6 @@
 /*
 %%% portions-copyright-cmetz-96
-Portions of this software are Copyright 1996-1997 by Craig Metz, All Rights
+Portions of this software are Copyright 1996-1998 by Craig Metz, All Rights
 Reserved. The Inner Net License Version 2 applies to these portions of
 the software.
 You should have received a copy of the license with this software. If
@@ -73,6 +73,10 @@ static char sccsid[] = "@(#)ftp.c	8.6 (Berkeley) 10/27/94";
 
 #include "ftp_var.h"
 
+#if NETPROC
+#include <net/security.h>
+#endif /* NETPROC */
+
 #if INET6
 #include "support.h"
 #endif /* INET6 */
@@ -104,10 +108,11 @@ FILE	*cin, *cout;
 typedef void (*sig_t)(int);
 #endif /* defined(__linux__) && !defined(__GLIBC__) */
 
-#if NETSEC
+#if NETPROC
 extern void *control_request, *data_request;
 extern int control_requestlen, data_requestlen;
-#endif /* NETSEC */
+int control;
+#endif /* NETPROC */
 
 char *
 hookup(host, port)
@@ -122,8 +127,7 @@ hookup(host, port)
   int i;
   struct addrinfo req, *ai, *ai2;
   int s, len, tos;
-  static char hostnamebuf[80];
-  char buf1[46], buf2[16];
+  static char hostnamebuf[NI_MAXHOST];
 
   memset(&req, 0, sizeof(struct addrinfo));
 
@@ -138,13 +142,14 @@ hookup(host, port)
     return NULL;
   }
 
-#if NETSEC
+#if NETPROC
   s = inner_connect(ai, control_request, control_requestlen, NULL, NULL, "ftp", &ai2);
-#else /* NETSEC */
+#else /* NETPROC */
   s = inner_connect(ai, NULL, 0, NULL, NULL, "ftp", &ai2);
-#endif /* NETSEC */
+#endif /* NETPROC */
   if (s >= 0) {
-    strcpy(hostname = hostnamebuf, ai2->ai_canonname);
+    strncpy(hostname = hostnamebuf, ai2->ai_canonname, sizeof(hostnamebuf)-1);
+    hostnamebuf[sizeof(hostnamebuf)-1] = 0;
     memcpy(&hisctladdr, ai2->ai_addr, ai2->ai_addrlen);
   };
   freeaddrinfo(ai);
@@ -229,6 +234,9 @@ hookup(host, port)
 #endif
 	cin = fdopen(s, "r");
 	cout = fdopen(s, "w");
+#if NETPROC
+        control = s;
+#endif /* NETPROC */
 	if (cin == NULL || cout == NULL) {
 		warnx("fdopen failed.");
 		if (cin)
@@ -1078,7 +1086,7 @@ initconn()
 	char *p, *a;
 	int result, len, tmpno = 0;
 	int on = 1;
-#ifndef INET6
+#if !INET6
 	int a0, a1, a2, a3, p0, p1;
 #endif /* INET6 */
 
@@ -1096,37 +1104,74 @@ initconn()
 		    setsockopt(data, SOL_SOCKET, SO_DEBUG, (char *)&on,
 			       sizeof (on)) < 0)
 			perror("ftp: setsockopt (ignored)");
-#if NETSEC
-                if (data_request)
-		  if (setsockopt(data, SOL_SOCKET, SO_SECURITY_REQUEST, data_request, data_requestlen) < 0)
-		    fprintf(stderr, "ftp: setsockopt(..., SO_SECURITY_REQUEST, data_request, ...): %s(%d)\n", strerror(errno), errno);
-#endif /* NETSEC */
+#if NETPROC
+                if (data_requestlen) {
+		  char *identity;
+                  int identtype;
+
+                  if (netproc_setrequest(data, 0, data_request, data_requestlen))
+		    fprintf(stderr, "ftp: netproc_setrequest: %s(%d)\n", strerror(errno), errno);
+
+#if 0
+                  if (net_security_getlocalident(control, &identtype, &identity)) {
+                    if (errno == ESRCH)
+                      goto skiplocal;
+                    fprintf(stderr, "ftp: net_security_getlocalident: %s(%d)\n", strerror(errno), errno);
+                  };
+                  if (net_security_setlocalident(data, identtype, identity))
+                    fprintf(stderr, "ftp: net_security_setlocalident: %s(%d)\n", strerror(errno), errno);
+
+skiplocal:
+                  if (net_security_getremoteident(control, &identtype, &identity)) {
+                    if (errno == ESRCH)
+                      goto skipremote;
+                    fprintf(stderr, "ftp: net_security_getremoteident: %s(%d)\n", strerror(errno), errno);
+                  };
+                  if (net_security_setremoteident(data, identtype, identity))
+                    fprintf(stderr, "ftp: net_security_setremoteident: %s(%d)\n", strerror(errno), errno);
+                };
+skipremote:
+#else /* 0 */
+                };
+#endif /* 0 */
+#endif /* NETPROC */
 #if INET6
 		switch (howport) {
 		  case 0:
 		  case 1:
-		    result = command("SPASV");
-		    if ((result == COMPLETE) && !sporttosa(&data_addr, pasv, &hisctladdr)) {
-		      howport = 1;
+		    if (command("EPSV") == COMPLETE) {
+		      char *c, *c2;
+
+                      if ((c = strchr(reply_string, '(')) && (c2 = strchr(++c, ')'))) {
+			*c2 = 0;
+		        if (!eporttosa(&data_addr, c, &hisctladdr, 1)) {
+		          howport = 1;
+		          if ((passivemode != 2) && (command("EPSV ALL") == COMPLETE))
+		            passivemode = 2;
+		          break;
+			};
+		      };
+		    };
+		    if (howport)
+		      goto bad;
+		  case 2:
+		    if ((hisctladdr.sa.sa_family == AF_INET) && (myctladdr.sa.sa_family == AF_INET) && (command("PASV") == COMPLETE) && !porttosa(&data_addr, pasv, &hisctladdr)) {
+		      howport = 2;
 		      break;
 		    }
 		    if (howport)
 		      goto bad;
-		  case 2:
-		    if ((hisctladdr.sa.sa_family == AF_INET) && (myctladdr.sa.sa_family == AF_INET)) {
-		      result = command("PASV");
-		      if ((result == COMPLETE) && !porttosa(&data_addr, pasv, &hisctladdr)) {
-			howport = 2;
-			break;
-		      }
+		  case 3:
+		    if ((command("SPASV") == COMPLETE) && !sporttosa(&data_addr, pasv, &hisctladdr)) {
+		      howport = 3;
+		      break;
 		    }
 		    if (howport)
 		      goto bad;
 #if !INNER
-		  case 3:
-		    result = command("LPSV");
-		    if ((result == COMPLETE) && !lporttosa(&data_addr, pasv, &hisctladdr)) {
-		      howport = 3;
+		  case 4:
+		    if ((command("LPSV") == COMPLETE) && !lporttosa(&data_addr, pasv, &hisctladdr)) {
+		      howport = 4;
 		      break;
 		    }
 		    if (howport)
@@ -1136,7 +1181,7 @@ initconn()
 		    goto bad;
 		}
 
-		if (connect(data, (struct sockaddr *)&data_addr, NRL_SA_LEN((struct sockaddr *)&data_addr)) < 0) {
+		if (connect(data, (struct sockaddr *)&data_addr, SA_LEN((struct sockaddr *)&data_addr)) < 0) {
 		  perror("ftp: connect");
 		  goto bad;
 		}
@@ -1195,7 +1240,7 @@ initconn()
 
 noport:
 #if INET6
-  memcpy(&data_addr, &myctladdr, NRL_SA_LEN((struct sockaddr *)&myctladdr));
+  memcpy(&data_addr, &myctladdr, SA_LEN((struct sockaddr *)&myctladdr));
   if (sendport)
      switch(myctladdr.sa.sa_family) {
        case AF_INET:
@@ -1225,18 +1270,44 @@ noport:
 			sendport = 1;
 		return (1);
 	}
-#if NETSEC
-        if (data_request)
-	  if (setsockopt(data, SOL_SOCKET, SO_SECURITY_REQUEST, data_request, data_requestlen) < 0)
-	    fprintf(stderr, "ftp: setsockopt(..., SO_SECURITY_REQUEST, data_request, ...): %s(%d)\n", strerror(errno), errno);
-#endif /* NETSEC */
+#if NETPROC
+        if (data_requestlen) {
+          char *identity;
+          int identtype;
+
+          if (netproc_setrequest(data, 0, data_request, data_requestlen))
+	    fprintf(stderr, "ftp: netproc_setrequest: %s(%d)\n", strerror(errno), errno);
+
+#if 0
+          if (net_security_getlocalident(control, &identtype, &identity)) {
+            if (errno == ESRCH)
+              goto skiplocal_pasv;
+            fprintf(stderr, "ftp: net_security_getlocalident: %s(%d)\n", strerror(errno), errno);
+          };
+          if (net_security_setlocalident(data, identtype, identity))
+            fprintf(stderr, "ftp: net_security_setlocalident: %s(%d)\n", strerror(errno), errno);
+
+skiplocal_pasv:
+          if (net_security_getremoteident(control, &identtype, &identity)) {
+            if (errno == ESRCH)
+              goto skipremote_pasv;
+            fprintf(stderr, "ftp: net_security_getremoteident: %s(%d)\n", strerror(errno), errno);
+          };
+          if (net_security_setremoteident(data, identtype, identity))
+            fprintf(stderr, "ftp: net_security_setremoteident: %s(%d)\n", strerror(errno), errno);
+        };
+skipremote_pasv:
+#else /* 0 */
+        };
+#endif /* 0 */
+#endif /* NETPROC */
 	if (!sendport)
 		if (setsockopt(data, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof (on)) < 0) {
 			warn("setsockopt (reuse address)");
 			goto bad;
 		}
 #if INET6
-	if (bind(data, (struct sockaddr *)&data_addr, NRL_SA_LEN((struct sockaddr *)&data_addr)) < 0) {
+	if (bind(data, (struct sockaddr *)&data_addr, SA_LEN((struct sockaddr *)&data_addr)) < 0) {
 #else /* INET6 */
 	if (bind(data, (struct sockaddr *)&data_addr, sizeof (data_addr)) < 0) {
 #endif /* INET6 */
@@ -1255,43 +1326,34 @@ noport:
 		warn("listen");
 	if (sendport) {
 #if INET6
-	  char buffer[80];
-
 	  switch (howport) {
 	    case 0:
 	    case 1:
-	      if (satosport(buffer, &data_addr)) {
+	      {
+	      char buf[1+1+1+NI_MAXHOST+1+NI_MAXSERV+1+1];
+
+	      if (satoeport(buf, sizeof(buf), &data_addr, 0)) {
 		printf("Can't convert address for transmission\n");
 		goto bad;
 	      }
-	      result = command("SPORT %s", buffer);
-#if 0
-	      if (result == ERROR && sendport == -1) {
-		sendport = 0;
-		tmpno = 1;
-		goto noport;
-	      }
-#endif /* 0 */
+	      result = command("EPRT %s", buf);
 	      if (result == COMPLETE) {
 		howport = 1;
 		return 0;
 	      }
 	      if (howport)
 		return -1;
+              };
 	    case 2:
+              {
+              char buf[24];
+
 	      if ((hisctladdr.sa.sa_family == AF_INET) && (myctladdr.sa.sa_family == AF_INET)) {
-		if (satoport(buffer, &data_addr)) {
+		if (satoport(buf, sizeof(buf), &data_addr)) {
 		  printf("Can't convert address for transmission\n");
 		  goto bad;
 		}
-		result = command("PORT %s", buffer);
-#if 0
-		if (result == ERROR && sendport == -1) {
-			sendport = 0;
-			tmpno = 1;
-			goto noport;
-		}
-#endif /* 0 */
+		result = command("PORT %s", buf);
 		if (result == COMPLETE) {
 		  howport = 2;
 		  return 0;
@@ -1299,19 +1361,40 @@ noport:
 	      }
 	      if (howport)
 		return -1;
-#if !INNER
-	    case 3:
-	      if (satolport(buffer, &data_addr)) {
+              };
+            case 3:
+	      {
+              char sbuf[NI_MAXSERV];
+
+	      if (satosport(sbuf, sizeof(sbuf), &data_addr)) {
 		printf("Can't convert address for transmission\n");
 		goto bad;
 	      }
-	      result = command("LPRT %s", buffer);
+	      result = command("SPORT %s", sbuf);
 	      if (result == COMPLETE) {
 		howport = 3;
 		return 0;
 	      }
 	      if (howport)
 		return -1;
+              };
+#if !INNER
+	    case 4:
+              {
+              char buf[1+1+2+1+32+1+1+4+1];
+
+	      if (satolport(buf, sizeof(buf), &data_addr)) {
+		printf("Can't convert address for transmission\n");
+		goto bad;
+	      }
+	      result = command("LPRT %s", buf);
+	      if (result == COMPLETE) {
+		howport = 4;
+		return 0;
+	      }
+	      if (howport)
+		return -1;
+              };
 #endif /* !INNER */
 	    default:
 	      return 1;
@@ -1501,10 +1584,10 @@ pswitch(flag)
 		ip->name[0] = 0;
 	hostname = op->name;
 #if INET6
-	memcpy(&ip->hctl, &hisctladdr, NRL_SA_LEN((struct sockaddr *)&hisctladdr));
-	memcpy(&hisctladdr, &op->hctl, NRL_SA_LEN((struct sockaddr *)&op->hctl));
-	memcpy(&ip->mctl, &myctladdr, NRL_SA_LEN((struct sockaddr *)&myctladdr));
-	memcpy(&myctladdr, &op->mctl, NRL_SA_LEN((struct sockaddr *)&op->mctl));
+	memcpy(&ip->hctl, &hisctladdr, SA_LEN((struct sockaddr *)&hisctladdr));
+	memcpy(&hisctladdr, &op->hctl, SA_LEN((struct sockaddr *)&op->hctl));
+	memcpy(&ip->mctl, &myctladdr, SA_LEN((struct sockaddr *)&myctladdr));
+	memcpy(&myctladdr, &op->mctl, SA_LEN((struct sockaddr *)&op->mctl));
 #else /* INET6 */
 	ip->hctl = hisctladdr;
 	hisctladdr = op->hctl;
