@@ -1,6 +1,6 @@
 /*
 %%% portions-copyright-cmetz-96
-Portions of this software are Copyright 1996-1997 by Craig Metz, All Rights
+Portions of this software are Copyright 1996-1998 by Craig Metz, All Rights
 Reserved. The Inner Net License Version 2 applies to these portions of
 the software.
 You should have received a copy of the license with this software. If
@@ -58,6 +58,7 @@ static char sccsid[] = "@(#)ftpd.c	8.5 (Berkeley) 4/28/95";
 #include <sys/ioctl.h>
 #include <sys/socket.h>
 #include <sys/wait.h>
+#include <sys/cdefs.h>
 
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
@@ -86,6 +87,10 @@ static char sccsid[] = "@(#)ftpd.c	8.5 (Berkeley) 4/28/95";
 #include <time.h>
 #include <unistd.h>
 
+#if NETPROC
+#include <net/security.h>
+#endif /* NETPROC */
+
 #include "pathnames.h"
 #include "extern.h"
 
@@ -118,10 +123,10 @@ struct	sockaddr_in his_addr;
 struct	sockaddr_in pasv_addr;
 #endif /* INET6 */
 
-#ifdef NETSEC
+#ifdef NETPROC
 void *request = NULL;
 int requestlen = 0;
-#endif /* NETSEC */
+#endif /* NETPROC */
 
 int	data;
 jmp_buf	errcatch, urgcatch;
@@ -167,6 +172,10 @@ char	*LastArgv = NULL;	/* end of argv */
 char	proctitle[LINE_MAX];	/* initial part of title */
 #endif /* SETPROCTITLE */
 
+#ifdef HIDE
+char buffer[1024];
+#endif /* HIDE */
+
 #define LOGCMD(cmd, file) \
 	if (logging > 1) \
 	    syslog(LOG_INFO,"%s %s%s", cmd, \
@@ -273,7 +282,7 @@ main(argc, argv, envp)
 #endif /* INET6 */
 #endif
 #if INET6
-	memcpy(&data_source, &ctrl_addr, NRL_SA_LEN((struct sockaddr *)&ctrl_addr));
+	memcpy(&data_source, &ctrl_addr, SA_LEN((struct sockaddr *)&ctrl_addr));
 	switch(ctrl_addr.sa.sa_family) {
 	  case AF_INET:
 	    data_source.sin.sin_port = htons(ntohs(ctrl_addr.sin.sin_port) - 1);
@@ -307,11 +316,11 @@ main(argc, argv, envp)
 	LastArgv = envp[-1] + strlen(envp[-1]);
 #endif /* SETPROCTITLE */
 
-#ifdef NETSEC
+#ifdef NETPROC
 	while ((ch = getopt(argc, argv, "dlt:T:u:vS:")) != EOF) {
-#else /* NETSEC */
+#else /* NETPROC */
 	while ((ch = getopt(argc, argv, "dlt:T:u:v")) != EOF) {
-#endif /* NETSEC */
+#endif /* NETPROC */
 		switch (ch) {
 		case 'd':
 			debug = 1;
@@ -349,14 +358,14 @@ main(argc, argv, envp)
 			debug = 1;
 			break;
 
-#ifdef NETSEC
+#ifdef NETPROC
 		case 'S':
-                        if (netsec_strtoreq(optarg, &request, &requestlen)) {
-                          fprintf(stderr, "ftpd: netsec_strtoreq(%s) failed\n", optarg);
+                        if (netproc_argstrtorequest(optarg, &request, &requestlen)) {
+                          fprintf(stderr, "ftpd: netprog_argstrtorequest(%s) failed\n", optarg);
                           exit(1);
                         };
 			break;
-#endif /* NETSEC */
+#endif /* NETPROC */
 
 		default:
 			warnx("unknown flag -%c ignored", optopt);
@@ -382,19 +391,25 @@ main(argc, argv, envp)
 #else /* INET6 */
 	dolog(&his_addr);
 #endif /* INET6 */
-#if NETSEC
-        if (!request) {
-          requestlen = 2048; /* XXX */
-          if (!(request = malloc(requestlen))) {
-	    syslog(LOG_ERR, "malloc(%d): %s(%d)", requestlen, strerror(errno), errno);
-            exit(1);
-          };
-          if (getsockopt(0, SOL_SOCKET, SO_SECURITY_REQUEST, request, &requestlen) < 0) {
-	    syslog(LOG_ERR, "getsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)", strerror(errno), errno);
+#if NETPROC
+        if (!requestlen) {
+          int num;
+
+          if (netproc_getnum(0, &num)) {
+            if (errno == ENOPROTOOPT)
+	      goto skip;      
+	    syslog(LOG_ERR, "netproc_getnum(0, ...): %s(%d)", strerror(errno), errno);
 	    exit(1);
           };
+
+          if (netproc_getrequest(0, num, request, &requestlen)) {
+	    syslog(LOG_ERR, "netproc_getrequest(0, %d, ...): %s(%d)", num, strerror(errno), errno);
+	    exit(1);
+          };
+
+skip:
         };
-#endif /* NETSEC */
+#endif /* NETPROC */
 	/*
 	 * Set up default state
 	 */
@@ -912,12 +927,44 @@ getdatasock(mode)
 #endif /* INET6 */
 	if (s < 0)
 		goto bad;
-#ifdef NETSEC
-	if (setsockopt(s, SOL_SOCKET, SO_SECURITY_REQUEST, request, requestlen) < 0) {
-	  syslog(LOG_ERR, "setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)\n", strerror(errno), errno);
-          goto bad;
+#ifdef NETPROC
+	if (requestlen) {
+          char *identity;
+          int identtype;
+
+	  if (netproc_setrequest(s, 0, request, requestlen)) {
+	    syslog(LOG_ERR, "netproc_setrequest: %s(%d)\n", strerror(errno), errno);
+            goto bad;
+          };
+
+#if 0
+          if (net_security_getlocalident(s, &identtype, &identity)) {
+            if (errno == ESRCH)
+              goto skiplocal;
+            syslog(LOG_ERR, "net_security_getlocalident: %s(%d)\n", strerror(errno), errno);
+            goto bad;
+          };
+          if (netproc_setlocalident(data, identtype, identity)) {
+            syslog(LOG_ERR, "netproc_setlocalident: %s(%d)\n", strerror(errno), errno);
+            goto bad;
+          };
+
+skiplocal:
+          if (net_security_getremoteident(s, &identtype, &identity)) {
+            if (errno == ESRCH)
+              goto skipremote;
+            syslog(LOG_ERR, "net_security_getremoteident: %s(%d)\n", strerror(errno), errno);
+            goto bad;
+          };
+          if (netproc_setremoteident(data, identtype, identity)) {
+            syslog(LOG_ERR, "netproc_setremoteident: %s(%d)\n", strerror(errno), errno);
+            goto bad;
+          };
+#endif /* 0 */
         };
-#endif /* NETSEC */
+
+skipremote:
+#endif /* NETPROC */
 	if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR,
 	    (char *) &on, sizeof(on)) < 0)
 		goto bad;
@@ -929,7 +976,7 @@ getdatasock(mode)
 	for (tries = 1; ; tries++) {
 		if (bind(s, (struct sockaddr *)&data_source,
 #if INET6
-		    NRL_SA_LEN((struct sockaddr *)&data_source)) >= 0)
+		    SA_LEN((struct sockaddr *)&data_source)) >= 0)
 #else /* INET6 */
 		    sizeof(data_source)) >= 0)
 #endif /* INET6 */
@@ -1018,7 +1065,7 @@ dataconn(name, size, mode)
 	}
 	if (usedefault)
 #if INET6
-	  memcpy(&data_dest, &his_addr, NRL_SA_LEN((struct sockaddr *)&his_addr));
+	  memcpy(&data_dest, &his_addr, SA_LEN((struct sockaddr *)&his_addr));
 #else /* INET6 */
 		data_dest = his_addr;
 #endif /* INET6 */
@@ -1027,7 +1074,7 @@ dataconn(name, size, mode)
 	if (file == NULL) {
 #if INET6
 	  char addr[NI_MAXHOST], service[NI_MAXSERV];
-	  if (getnameinfo((struct sockaddr *)&data_source, NRL_SA_LEN((struct sockaddr *)&data_source), addr, sizeof(addr), service, sizeof(service), NI_NUMERICHOST | NI_NUMERICSERV))
+	  if (getnameinfo((struct sockaddr *)&data_source, SA_LEN((struct sockaddr *)&data_source), addr, sizeof(addr), service, sizeof(service), NI_NUMERICHOST | NI_NUMERICSERV))
 	    reply(425, "Can't create data socket: %s.");
 	  else
 	    reply(425, "Can't create data socket (%s,%s): %s.", addr, service, strerror(errno));
@@ -1041,7 +1088,7 @@ dataconn(name, size, mode)
 	data = fileno(file);
 	while (connect(data, (struct sockaddr *)&data_dest,
 #if INET6
-	    NRL_SA_LEN((struct sockaddr *)&data_dest)) < 0) {
+	    SA_LEN((struct sockaddr *)&data_dest)) < 0) {
 #else /* INET6 */
 	    sizeof(data_dest)) < 0) {
 #endif /* INET6 */
@@ -1270,7 +1317,7 @@ statcmd()
 #if INET6
         {
 	  char host[NI_MAXHOST];
-	  if (!getnameinfo((struct sockaddr *)&his_addr, NRL_SA_LEN((struct sockaddr *)&his_addr), host, sizeof(host), NULL, 0, NI_NUMERICHOST))
+	  if (!getnameinfo((struct sockaddr *)&his_addr, SA_LEN((struct sockaddr *)&his_addr), host, sizeof(host), NULL, 0, NI_NUMERICHOST))
 	    printf(" (%s)", host);
         }
 #else /* INET6 */
@@ -1321,9 +1368,12 @@ statcmd()
 printaddr:
 #if INET6
 		{
-		  char buf[128];
-		  if (!satosport(buf, sa))
-		    printf(" (%s)", buf);
+		  char sbuf[NI_MAXSERV];
+
+		  if (getnameinfo(sa, SA_LEN(sa), NULL, 0, sbuf, sizeof(sbuf), NI_NUMERICSERV))
+		    printf(" (???)");
+		  else
+		    printf(" (%s)", sbuf);
 		  printf("\r\n");
 		}
 #else /* INET6 */
@@ -1366,9 +1416,28 @@ reply(n, fmt, va_alist)
 #else
 	va_start(ap);
 #endif
+#ifdef HIDE
+	{
+	  int i = 0;
+	  char *c;
+
+	  if ((sizeof(buffer) - i) > 0)
+	    i += snprintf(buffer + i, sizeof(buffer) - i, "%d ", n);
+	  if ((sizeof(buffer) - i) > 0)
+	    i += vsnprintf(buffer + i, sizeof(buffer) - i, fmt, ap);
+	  buffer[sizeof(buffer)-1] = 0;
+
+	  c = buffer;
+	  while(*c)
+	    printf("%c%c%c", IAC, NOP, *c++);
+
+	  printf("%c%c\r%c%c\n%c%c", IAC, NOP, IAC, NOP, IAC, NOP);
+	};
+#else /* HIDE */
 	(void)printf("%d ", n);
 	(void)vprintf(fmt, ap);
 	(void)printf("\r\n");
+#endif /* HIDE */
 	(void)fflush(stdout);
 	if (debug) {
 		syslog(LOG_DEBUG, "<--- %d ", n);
@@ -1496,7 +1565,7 @@ pwd()
 {
 	char path[MAXPATHLEN + 1];
 
-	if (getwd(path) == (char *)NULL)
+	if (getcwd(path, sizeof(path)) == (char *)NULL)
 		reply(550, "%s.", path);
 	else
 		reply(257, "\"%s\" is current directory.", path);
@@ -1538,7 +1607,7 @@ dolog(sin)
 #endif /* INET6 */
 {
 #if INET6
-  if (getnameinfo(sa, NRL_SA_LEN(sa), remotehost, sizeof(remotehost), NULL, 0, NI_NUMERICHOST | NI_NUMERICSERV)) {
+  if (getnameinfo(sa, SA_LEN(sa), remotehost, sizeof(remotehost), NULL, 0, NI_NUMERICHOST | NI_NUMERICSERV)) {
     syslog(LOG_ERR, "getnameinfo failed, bailing");
     exit(1);
   }
@@ -1616,8 +1685,7 @@ myoob(signo)
  */
 void
 #if INET6
-passive(l)
-  int l;
+passive(int how, int af)
 #else /* INET6 */
 passive()
 #endif /* INET6 */
@@ -1626,7 +1694,7 @@ passive()
 	char *p, *a;
 
 #if INET6
-	pdata = socket(ctrl_addr.sa.sa_family, SOCK_STREAM, 0);
+	pdata = socket(af, SOCK_STREAM, 0);
 #else /* INET6 */
 	pdata = socket(AF_INET, SOCK_STREAM, 0);
 #endif /* INET6 */
@@ -1634,24 +1702,31 @@ passive()
 		perror_reply(425, "Can't open passive connection");
 		return;
 	}
-#ifdef NETSEC
-	if (setsockopt(pdata, SOL_SOCKET, SO_SECURITY_REQUEST, request, requestlen) < 0) {
-	  syslog(LOG_ERR, "setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)\n", strerror(errno), errno);
+#ifdef NETPROC
+	if (requestlen && netproc_setrequest(pdata, 0, request, requestlen)) {
+	  syslog(LOG_ERR, "netproc_setrequest: %s(%d)\n", strerror(errno), errno);
           goto pasv_error;
         };
-#endif /* NETSEC */
+#endif /* NETPROC */
 #if INET6
-	memcpy(&pasv_addr, &ctrl_addr, NRL_SA_LEN((struct sockaddr *)&ctrl_addr));
-	switch(ctrl_addr.sa.sa_family) {
-	  case AF_INET:
-	    pasv_addr.sin.sin_port = 0;
-	    break;
-	  case AF_INET6:
-	    pasv_addr.sin6.sin6_port = 0;
-	    break;
-	  default:
-	    goto pasv_error;
-	}
+	{
+	struct addrinfo *ai, req;
+	char hbuf[NI_MAXHOST];
+
+	if (getnameinfo((struct sockaddr *)&ctrl_addr, SA_LEN((struct sockaddr *)&ctrl_addr), hbuf, sizeof(hbuf), NULL, 0, NI_NUMERICHOST))
+	  goto pasv_error;
+
+	memset(&req, 0, sizeof(struct addrinfo));
+	req.ai_family = af;
+	req.ai_socktype = SOCK_STREAM;
+	req.ai_flags = AI_PASSIVE;
+
+        if (getaddrinfo(hbuf, NULL, &req, &ai))
+	  goto pasv_error;
+
+	memcpy(&pasv_addr, ai->ai_addr, ai->ai_addrlen);
+	freeaddrinfo(ai);
+	};
 #else /* INET6 */
 	pasv_addr = ctrl_addr;
 	pasv_addr.sin_port = 0;
@@ -1659,7 +1734,7 @@ passive()
 	disable_signalling();
 	(void) seteuid((uid_t)0);
 #if INET6
-	if (bind(pdata, (struct sockaddr *)&pasv_addr, NRL_SA_LEN((struct sockaddr *)&pasv_addr)) < 0) {
+	if (bind(pdata, (struct sockaddr *)&pasv_addr, SA_LEN((struct sockaddr *)&pasv_addr)) < 0) {
 #else /* INET6 */
 	if (bind(pdata, (struct sockaddr *)&pasv_addr, sizeof(pasv_addr)) < 0) {
 #endif /* INET6 */
@@ -1675,30 +1750,67 @@ passive()
 	if (listen(pdata, 1) < 0)
 		goto pasv_error;
 #if INET6
-	{
-	  char buffer[80];
+	switch(how) {
+	  case 1:
+	    {
+	      char buf[1+1+1+NI_MAXHOST+1+NI_MAXSERV+1+1];
 
-	  switch(l) {
-	    case 1:
-	      if (satosport(buffer, &pasv_addr))
-		goto pasv_error;
-	      break;
-	    case 2:
-	      if (satoport(buffer, &pasv_addr))
-		goto pasv_error;
-	      break;
+	      if (satoeport(buf, sizeof(buf), &pasv_addr, 1)) {
+                close(pdata);
+                pdata = -1;
+                reply(425, "Can't open passive connection");
+                return;
+              };
+	      reply(228, "Entering Passive Mode (%s)", buf);
+	    };
+	    break;
+	  case 2:
+	    {
+	      char buf[24];
+
+	      if (satoport(buf, sizeof(buf), &pasv_addr)) {
+                close(pdata);
+                pdata = -1;
+                reply(425, "Can't open passive connection");
+                return;
+              };
+	      reply(227, "Entering Passive Mode (%s)", buf);
+	    };
+	    break;
+	  case 3:
+	    {
+              char sbuf[NI_MAXSERV];
+
+	      if (satosport(sbuf, sizeof(sbuf), &pasv_addr)) {
+                close(pdata);
+                pdata = -1;
+                reply(425, "Can't open passive connection");
+                return;
+              }
+              reply(229, "Entering Passive Mode (%s)", sbuf);
+	    };
+	    break;
 #if !INNER
-	    case 3:
-	      if (satolport(buffer, &pasv_addr))
-		goto pasv_error;
-	      break;
+	  case 4:
+	    {
+	      char buf[1+1+2+1+32+1+1+4+1];
+	      if (satolport(buf, sizeof(buf), &pasv_addr)) {
+                close(pdata);
+                pdata = -1;
+                reply(425, "Can't open passive connection");
+                return;
+              };
+	      reply(228, "Entering Passive Mode (%s)", buf);
+	    };
+	    break;
 #endif /* !INNER */
-	    default:
-	      goto pasv_error;
-	  }
-	  reply(227, "Entering Passive Mode (%s)", buffer);
-	  return;
-	}
+	  default:
+            close(pdata);
+            pdata = -1;
+            reply(425, "Can't open passive connection");
+            return;
+	};
+	return;
 #else /* INET6 */
 	a = (char *) &pasv_addr.sin_addr;
 	p = (char *) &pasv_addr.sin_port;
