@@ -1,6 +1,6 @@
 /*
 %%% portions-copyright-cmetz-96
-Portions of this software are Copyright 1996-1997 by Craig Metz, All Rights
+Portions of this software are Copyright 1996-1998 by Craig Metz, All Rights
 Reserved. The Inner Net License Version 2 applies to these portions of
 the software.
 You should have received a copy of the license with this software. If
@@ -202,6 +202,10 @@ static char sccsid[] = "@(#)inetd.c	8.4 (Berkeley) 4/13/94";
 #include <syslog.h>
 #include <unistd.h>
 
+#if NETSEC
+#include <net/security.h>
+#endif /* NETSEC */
+
 #if INET6
 #include "support.h"
 #endif /* INET6 */
@@ -633,17 +637,15 @@ config(signo)
 				SWAP(sep->se_argv[i], cp->se_argv[i]);
 
 #ifdef NETSEC
-			if ((cp->se_requestlen != sep->se_requestlen) || (cp->se_request && memcmp(cp->se_request, sep->se_request, cp->se_requestlen))) {
-			  if (sep->se_request)
-			    free(sep->se_request);
-			  sep->se_request = cp->se_request;
+			if ((cp->se_requestlen != sep->se_requestlen) || (memcmp(cp->se_request, sep->se_request, cp->se_requestlen))) {
+			  memcpy(sep->se_request, cp->se_request, cp->se_requestlen);
 			  sep->se_requestlen = cp->se_requestlen;
 
-			  if (sep->se_request && (sep->se_fd != -1))
-			    if (setsockopt(sep->se_fd, SOL_SOCKET, SO_SECURITY_REQUEST, sep->se_request, sep->se_requestlen) < 0) {
-			      syslog(LOG_ERR, "setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)", strerror(errno), errno);
+			  if (sep->se_requestlen && (sep->se_fd != -1))
+			    if (net_security_setrequest(sep->se_fd, 0, sep->se_request, sep->se_requestlen)) {
+			      syslog(LOG_ERR, "net_security_setrequest: %s(%d)", strerror(errno), errno);
 			      if (debug)
-				fprintf(stderr, "setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)\n", strerror(errno), errno);
+				fprintf(stderr, "net_security_setrequest: %s(%d)\n", strerror(errno), errno);
 			      exit(1);
 			    };
 			};
@@ -756,19 +758,17 @@ setsockopt(fd, SOL_SOCKET, opt, (char *)&on, sizeof (on))
 #endif /* OFFSETPORT */
 
 #ifdef NETSEC
-	if (sep->se_request && (setsockopt(sep->se_fd, SOL_SOCKET, SO_SECURITY_REQUEST, sep->se_request, sep->se_requestlen) < 0)) {
-	  if (setsockopt(sep->se_fd, SOL_SOCKET, SO_SECURITY_REQUEST, sep->se_request, sep->se_requestlen) < 0) {
-	    syslog(LOG_ERR, "setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)", strerror(errno), errno);
-	    if (debug)
-	      fprintf(stderr, "setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)\n", strerror(errno), errno);
-	    exit(1);
-	  };
+	if (sep->se_requestlen && net_security_setrequest(sep->se_fd, 0, sep->se_request, sep->se_requestlen)) {
+	  syslog(LOG_ERR, "net_security_setrequest: %s(%d)", strerror(errno), errno);
+	  if (debug)
+	    fprintf(stderr, "net_security_setrequest: %s(%d)\n", strerror(errno), errno);
+	  exit(1);
 	};
 #endif /* NETSEC */
 
 	if (bind(sep->se_fd, (struct sockaddr *)&sep->se_ctrladdr,
 #if INET6
-	  NRL_SA_LEN((struct sockaddr *)&sep->se_ctrladdr)) < 0) {
+	  SA_LEN((struct sockaddr *)&sep->se_ctrladdr)) < 0) {
 #else /* INET6 */
 	    sizeof (sep->se_ctrladdr)) < 0) {
 #endif /* INET6 */
@@ -992,12 +992,9 @@ more:
 	{
 	  char *c = newstr(sskip(&cp));
 
-	  sep->se_request = NULL;
-	  sep->se_requestlen = 0;
-
 	  if (strcmp(c, "*")) {
-	    if (netsec_strtoreq(c, &sep->se_request, &sep->se_requestlen)) {
-	      syslog(LOG_ERR, "netsec_strtoreq(%s) failed", c);
+	    if (net_security_strtorequest(c, &sep->se_request, &sep->se_requestlen)) {
+	      syslog(LOG_ERR, "net_security_strtorequest(%s, ...) failed", c);
 	      exit(1);
 	    };
 	  };
@@ -1436,11 +1433,11 @@ print_service(action, sep)
 	struct servtab *sep;
 {
 #if NETSEC
-        char buffer[64];
+        char buffer[NET_SECURITY_REQUESTSTR_MAX];
 
-	if (sep->se_request) {
-	  if (nrl_reqtostr(sep->se_request, sep->se_requestlen, buffer, sizeof(buffer))) {
-	    fprintf(stderr, "nrl_reqtostr failed\n");
+	if (sep->se_requestlen) {
+	  if (net_security_requesttostr(sep->se_request, sep->se_requestlen, buffer, sizeof(buffer))) {
+	    fprintf(stderr, "net_security_requesttostr failed\n");
 	    exit(1);
 	  };
 	} else
