@@ -1,6 +1,6 @@
 /*
 %%% portions-copyright-cmetz-96
-Portions of this software are Copyright 1996-1997 by Craig Metz, All Rights
+Portions of this software are Copyright 1996-1998 by Craig Metz, All Rights
 Reserved. The Inner Net License Version 2 applies to these portions of
 the software.
 You should have received a copy of the license with this software. If
@@ -96,6 +96,10 @@ Research Laboratory (NRL).
 #include <setjmp.h>
 #endif /* FASTCTO */
 
+#if NETPROC
+#include <net/netproc.h>
+#endif /* NETPROC */
+
 #include "support.h"
 
 extern char *optarg;
@@ -122,11 +126,12 @@ typedef union {
 char addrbuf[sizeof(su)], addrbuf2[sizeof(su)];
 int addrlen;
 
+#undef IPV6_PKTINFO
 #if INET6 && defined(IPV6_PKTINFO)
 struct in6_pktinfo in6_pktinfo;
 #endif /* INET6 && defined(IPV6_PKTINFO) */
 
-char controlbuf[CMSG_SPACE(sizeof(struct in6_pktinfo))];
+char *controlbuf;
 
 char *datablock;
 int datablocklen = 65535;
@@ -135,12 +140,15 @@ char messagebuf[1024];
 
 char *myname;
 
-char *requeststr = NULL;
-
 #if INET6 && defined(IPV6_ADDRFORM)
 int aform = 0;
 #endif /* INET6 && defined(IPV6_ADDRFORM) */
 
+#if NETPROC
+void *request = NULL;
+int requestlen = 0;
+#endif /* NETPROC */
+
 #ifdef FASTCTO
 static jmp_buf timeout_env;
 
@@ -206,9 +214,9 @@ usage(char *myname)
 #ifdef ADDRFORM
   fprintf(stderr, " [-f address_form]");
 #endif /* ADDRFORM */
-#if NETSEC
+#if NETPROC
   fprintf(stderr, " [-S security_request]");
-#endif /* NETSEC */
+#endif /* NETPROC */
   fprintf(stderr," [-s] [host] [port]\n");
   exit(1);
 }
@@ -225,13 +233,15 @@ main(argc, argv)
   char hbuf[32], sbuf[8], dbuf[32], ibuf[16];
   struct msghdr msghdr, msghdr2;
   struct iovec iovec;
+  unsigned int ifindex = 0;
+  char *srcaddr = NULL;
  
   if (myname = strrchr(argv[0], '/'))
     *(myname++) = 0;
   else
     myname = argv[0];
 
-  while ((ch = getopt(argc, argv, "S:A:N:T:a:f:t:snp:b:")) != EOF)
+  while ((ch = getopt(argc, argv, "S:A:N:T:a:f:t:snp:b:i:l:")) != EOF)
     switch(ch) {
     case 'a':
       if ((af = nrl_afnametonum(optarg)) == -1) {
@@ -247,14 +257,14 @@ main(argc, argv)
       }
       break;
 #endif /* ADDRFORM */
-#if NETSEC
+#if NETPROC
     case 'S':
-      if (netsec_strtoreq(optarg, &request, &requestlen)) {
-	fprintf(stderr, "netsec_strtoreq(%s, ...) failed!\n", optarg);
+      if (netproc_argstrtorequest(optarg, &request, &requestlen)) {
+	fprintf(stderr, "socktest: netproc_argstrtorequest(%s, ...) failed!\n", optarg);
 	exit(1);
       };
       break;
-#endif /* NETSEC */
+#endif /* NETPROC */
     case 's':
       server = 1;
       break;
@@ -279,6 +289,15 @@ main(argc, argv)
         exit(1);
       }
       break;
+    case 'i':
+      if (!(ifindex = if_nametoindex(optarg))) {
+	fprintf(stderr, "%s: %s: unknown/invalid interface name\n", myname, optarg);
+	exit(1);
+      };
+      break;
+    case 'l':
+      srcaddr = optarg;
+      break;
     default:
       usage(myname);
     }
@@ -295,26 +314,6 @@ main(argc, argv)
   if (!portname)
     portname = "7777";
 
-#if INET6 && defined(IPV6_PKTINFO)
-  if (optind < argc)
-    if (!(in6_pktinfo.ipi6_ifindex = if_nametoindex(argv[optind]))) {
-      fprintf(stderr, "%s: %s: unknown/invalid interface name\n", myname, argv[optind]);
-      exit(1);
-    };
-
-  optind++;
-
-  if (optind < argc) {
-    memset(&req, 0, sizeof(struct addrinfo));
-    req.ai_family = AF_INET6;
-    if (i = getaddrinfo(argv[optind], NULL, &req, &ai)) {
-      fprintf(stderr, "%s: getaddrinfo: %s.%s: %s(%d)\n", myname, argv[optind], NULL, gai_strerror(i), i);
-      exit(1);
-    };
-    in6_pktinfo.ipi6_addr = ((struct sockaddr_in6 *)(ai->ai_addr))->sin6_addr;
-  };
-#endif /* INET6 && defined(IPV6_PKTINFO) */
-
   memset(&req, 0, sizeof(struct addrinfo));
 
   req.ai_family = af;
@@ -344,46 +343,42 @@ main(argc, argv)
       continue;
     }
 
-#if NETSEC
-    if (requeststr) {
-      void *request, *request2;
-      int requestlen, requestlen2;
+#if NETPROC
+    if (requestlen) {
+      void *request2;
+      int requestlen2;
 
-      if (setsockopt(s, SOL_SOCKET, SO_SECURITY_REQUEST, request, requestlen) < 0) {
-	fprintf(stderr, "%s: setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)", myname, strerror(errno), errno);
+      if (netproc_setrequest(s, 0, request, requestlen)) {
+	fprintf(stderr, "%s: netproc_setrequest: %s(%d)\n", myname, strerror(errno), errno);
 	exit(1);
       };
 
-      if (!(request2 = malloc(requestlen2 = requestlen))) {
-	fprintf(stderr, "%s: malloc(%d): %s(%d)", myname, requestlen, strerror(errno), errno);
-	exit(1);
-      };
-
-      if (getsockopt(s, SOL_SOCKET, SO_SECURITY_REQUEST, request2, &requestlen2) < 0) {
-	fprintf(stderr, "%s: getsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)", myname, strerror(errno), errno);
+      if (netproc_getrequest(s, 0, &request2, &requestlen2)) {
+	fprintf(stderr, "%s: netproc_getrequest: %s(%d)\n", myname, strerror(errno), errno);
 	exit(1);
       };
 
       if ((requestlen != requestlen2) || memcmp(request, request2, requestlen)) {
-        fprintf(stderr, "%s: kernel changed request!\n", myname);
-        exit(1);
-      };
+	char *buffer, *buffer2;
 
-      {
-        char buffer[8];
-
-        if (nrl_reqtostr(request, requestlen, buffer, sizeof(buffer))) {
-          fprintf(stderr, "%s: nrl_reqtostr failed\n", myname);
+        if (netproc_requesttostr(request, requestlen, &buffer, sizeof(buffer))) {
+          fprintf(stderr, "%s: netproc_requesttostr failed\n", myname);
           exit(1);
         };
 
-        printf("%s: made security request of '%s'\n", myname, buffer);
-      };
+        printf("%s: warning: set security request to '%s',", myname, buffer);
+	free(buffer);
 
-      free(request2);
-      free(request);
+        if (netproc_requesttostr(request2, requestlen2, &buffer2, sizeof(buffer))) {
+          fprintf(stderr, "%s: netproc_requesttostr failed\n", myname);
+          exit(1);
+        };
+
+        printf("%s: warning: but got back\n%s\n", myname, buffer2);
+	free(buffer2);
+      };
     };
-#endif /* NETSEC */
+#endif /* NETPROC */
 
     if (ai->ai_flags & AI_PASSIVE) {
       if (bind(s, ai->ai_addr, ai->ai_addrlen) < 0) {
@@ -492,21 +487,54 @@ main(argc, argv)
     };
   };
 
-  if (ai->ai_family == AF_INET6) {
+  msghdr.msg_control = NULL;
+  msghdr.msg_controllen = 0;
+
+  if (ifindex || srcaddr) {
+    if (af != AF_INET6) {
+      fprintf(stderr, "%s: interface index and source address selection is currently only\n", myname);
+      fprintf(stderr, "for inet6 sockets.\n");
+      exit(1);
+    };
+
+#if INET6 && defined(IPV6_PKTINFO)
+    if (!(controlbuf = malloc(CMSG_SPACE(sizeof(struct in6_pktinfo))))) {
+      fprintf(stderr, "%s: can't allocate %d-byte buffer\n", myname, CMSG_SPACE(sizeof(struct in6_pktinfo)));
+      exit(1);
+    };
+
+    memset(controlbuf, 0, CMSG_SPACE(sizeof(struct in6_pktinfo)));
+
+    ((struct in6_pktinfo *)controlbuf)->ipi6_ifindex = ifindex;
+
+    if (srcaddr) {
+      struct addrinfo *ai3;
+
+      memset(&req, 0, sizeof(struct addrinfo));
+      req.ai_family = ai->ai_family;
+      if (i = getaddrinfo(srcaddr, NULL, &req, &ai3)) {
+	fprintf(stderr, "%s: getaddrinfo: %s.*: %s(%d)\n", myname, srcaddr, gai_strerror(i), i);
+	exit(1);
+      };
+
+      if (ai3->ai_next) {
+	fprintf(stderr, "%s: getaddrinfo: %s.* returned multiple addresses\n", myname, srcaddr);
+	exit(1);
+      };
+
+      ((struct in6_pktinfo *)controlbuf)->ipi6_addr = ((struct sockaddr_in6 *)(ai->ai_addr))->sin6_addr;
+    }
+
     msghdr.msg_control = controlbuf;
     msghdr.msg_controllen = CMSG_SPACE(sizeof(struct in6_pktinfo));
 
-    memset(controlbuf, 0, sizeof(controlbuf));
-
     ((struct cmsghdr *)controlbuf)->cmsg_len = CMSG_LEN(sizeof(struct in6_pktinfo));
     ((struct cmsghdr *)controlbuf)->cmsg_level = IPPROTO_IPV6;
     ((struct cmsghdr *)controlbuf)->cmsg_type = IPV6_PKTINFO;
 
     memcpy(CMSG_DATA((struct cmsghdr *)controlbuf), &in6_pktinfo, sizeof(struct in6_pktinfo));
-  } else {
-    msghdr.msg_control = NULL;
-    msghdr.msg_controllen = 0;
-  };
+#endif /* INET6 && defined(IPV6_PKTINFO) */
+  }
 
   if (patternlen)
     goto patterncode;
@@ -528,7 +556,7 @@ main(argc, argv)
     fflush(stdout);
 
     memcpy(&msghdr2, &msghdr, sizeof(struct msghdr));
-    msghdr2.msg_controllen = sizeof(controlbuf);
+    msghdr2.msg_controllen = CMSG_SPACE(sizeof(struct in6_pktinfo));
 
     if ((datasize = recvmsg(s, &msghdr2, 0)) < 0) {
       perror("recvmsg");
@@ -585,8 +613,17 @@ servercode:
   dbuf[0] = ibuf[0] = 0;
   msghdr.msg_name = addrbuf;
   msghdr.msg_namelen = sizeof(addrbuf);
-  msghdr.msg_control = controlbuf;
-  msghdr.msg_controllen = sizeof(controlbuf);
+#if INET6
+  if (ai->ai_family == AF_INET6) {
+    msghdr.msg_control = controlbuf;
+    msghdr.msg_controllen = CMSG_SPACE(sizeof(struct in6_pktinfo));
+  } else {
+#endif /* INET6 */
+    msghdr.msg_control = NULL;
+    msghdr.msg_controllen = 0;
+#if INET6
+  };
+#endif /* INET6 */
 
   while(1) {
     if (!datasize) {
