@@ -1,6 +1,6 @@
 /*
 %%% portions-copyright-cmetz-96
-Portions of this software are Copyright 1996-1997 by Craig Metz, All Rights
+Portions of this software are Copyright 1996-1998 by Craig Metz, All Rights
 Reserved. The Inner Net License Version 2 applies to these portions of
 the software.
 You should have received a copy of the license with this software. If
@@ -168,15 +168,15 @@ static char sccsid[] = "@(#)ping.c	8.3 (Berkeley) 4/28/95";
 #if INET6
 #include <sys/uio.h>
 
-#include <netinet/ipv6.h>
-#include <netinet/icmpv6.h>
+#include <netinet/ip6.h>
+#include <netinet/icmp6.h>
 
 #include "support.h"
 #endif /* INET6 */
 
-#if NETSEC
-#include <netsec/ipsec.h>
-#endif /* NETSEC */
+#if NETPROC
+#include <net/netproc.h>
+#endif /* NETPROC */
 
 #define	DEFDATALEN	(64 - 8)	/* default data length */
 #define	MAXIPLEN	60
@@ -234,7 +234,7 @@ int ident;			/* process id to identify our packets */
 int af = 0;                     /* address family */
 
 struct in6_pktinfo send_pktinfo;
-char send_controlbuf[CMSG_SPACE(sizeof(struct in6_pktinfo))];
+char *send_controlbuf;
 int send_controllen = 0;
 #endif /* INET6 */
 
@@ -274,15 +274,20 @@ main(argc, argv)
 	int ch, fdmask, hold, packlen, preload;
 	u_char *datap, *packet;
 	char *target, hnamebuf[MAXHOSTNAMELEN], *malloc();
-#if NETSEC
+#if NETPROC
 	void *request = NULL;
 	int requestlen = 0;
-#endif /* NETSEC */
+#endif /* NETPROC */
 #ifdef IP_OPTIONS
 	char rspace[3 + 4 * NROUTES + 1];	/* record route space */
 #endif
 	char tempbuf[64];
 
+	if (!(send_controlbuf = malloc(CMSG_SPACE(sizeof(struct in6_pktinfo))))) {
+	  fprintf(stderr, "ping: can't allocate %d-byte buffer\n", CMSG_SPACE(sizeof(struct in6_pktinfo)));
+	  exit(1);
+	};
+
 	preload = 0;
 	datap = &outpack[8 + sizeof(struct timeval)];
 #if INET6
@@ -292,9 +297,9 @@ main(argc, argv)
 #if INET6
 			    "a:I:"
 #endif /* INET6 */
-#if NETSEC
+#if NETPROC
 			    "S:"
-#endif /* NETSEC */
+#endif /* NETPROC */
 			    "Rc:dfa:hi:l:np:qrs:v")) != EOF)
 		switch(ch) {
 #if INET6
@@ -317,14 +322,14 @@ main(argc, argv)
 		  send_controllen = CMSG_SPACE(sizeof(struct in6_pktinfo));
 		  break;
 #endif /* INET6 */
-#if NETSEC
+#if NETPROC
 		case 'S':
-                  if (netsec_strtoreq(optarg, &request, &requestlen)) {
-                    fprintf(stderr, "ping: netsec_strtoreq(%s, ...) failed\n", optarg);
+                  if (netproc_argstrtorequest(optarg, &request, &requestlen)) {
+                    fprintf(stderr, "ping: netproc_argstrtorequest(%s, ...) failed\n", optarg);
                     exit(1);
                   };
                   break;
-#endif /* NETSEC */
+#endif /* NETPROC */
 		case 'c':
 			npackets = atoi(optarg);
 			if (npackets <= 0) {
@@ -408,6 +413,10 @@ main(argc, argv)
 	{
 	  int i;
 	  struct addrinfo req, *ai, *ai2;
+	  char *c;
+#if 0
+	  char hbuf[NI_MAXHOST];
+#endif /* 0 */
 
 	  memset(&req, 0, sizeof(struct addrinfo));
 	  req.ai_family = af;
@@ -417,18 +426,53 @@ main(argc, argv)
 	    exit(1);
 	  };
 
-          for (ai2 = ai; ai2 && (ai2->ai_family != AF_INET) && (ai2->ai_family != AF_INET6); ai2 = ai2->ai_next);
+	  ai2 = ai;
+
+          for (; ai2; ai2 = ai2->ai_next) {
+	    switch(ai2->ai_family) {
+	      case AF_INET:
+		c = "icmp";
+		break;
+	      case AF_INET6:
+		c = "icmp6";
+		break;
+	      default:
+		goto loop;
+	    };
+
+	    if (!(proto = getprotobyname(c))) {
+	      fprintf(stderr, "ping: %s: unknown protocol\n", c);
+	      continue;
+	    };
+
+#if 0
+	    if (getnameinfo(ai2->ai_addr, ai2->ai_addrlen, hbuf, sizeof(hbuf), NULL, 0, NI_NUMERICHOST)) {
+	      fprintf(stderr, "ping: getnameinfo failed\n");
+	      continue;
+	    };
+
+	    printf("Trying %s...\n", hbuf);
+#endif /* 0 */
+
+	    if ((s = socket(ai2->ai_family, SOCK_RAW, proto->p_proto)) < 0) {
+	      perror("ping: socket");
+	      continue;
+	    };
+
+	    break;
+loop:
+	  };
 
           if (!ai2) {
             fprintf(stderr, "ping: %s: no IP version 4 or 6 addresses available\n", hostname);
             exit(1);
           }; 
 
-	  memcpy(&whereto, ai->ai_addr, ai->ai_addrlen);
-	  af = ai->ai_family;
+	  memcpy(&whereto, ai2->ai_addr, ai2->ai_addrlen);
+	  af = ai2->ai_family;
 
           freeaddrinfo(ai);
-	}
+	};
 #else /* INET6 */
  	memset(&whereto, 0, sizeof(struct sockaddr));
 	to = (struct sockaddr_in *)&whereto;
@@ -474,33 +518,23 @@ main(argc, argv)
 
 	ident = getpid() & 0xFFFF;
 
-#if INET6
-        {
-	char *c = ((af == AF_INET6) ? "icmpv6" : "icmp");
-	if (!(proto = getprotobyname(c))) {
-		(void)fprintf(stderr, "ping: %s: unknown protocol\n", c);
-		exit(1);
-	}
-        }
-
-	if ((s = socket(af, SOCK_RAW, proto->p_proto)) < 0) {
-#else /* INET6 */
+#if !INET6
 	if (!(proto = getprotobyname("icmp"))) {
 		(void)fprintf(stderr, "ping: unknown protocol icmp.\n");
 		exit(1);
 	}
 	if ((s = socket(AF_INET, SOCK_RAW, proto->p_proto)) < 0) {
-#endif /* INET6 */
 		perror("ping: socket");
 		exit(1);
 	}
-#if NETSEC
-	if (request)
-          if (setsockopt(s, SOL_SOCKET, SO_SECURITY_REQUEST, request, requestlen) < 0) {
-            fprintf(stderr, "ftp: setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)\n", strerror(errno), errno);
+#endif /* !INET6 */
+#if NETPROC
+	if (requestlen)
+          if (netproc_setrequest(s, 0, request, requestlen)) {
+            fprintf(stderr, "ftp: netproc_setrequest: %s(%d)\n", strerror(errno), errno);
 	    exit(1);
 	  };
-#endif /* NETSEC */
+#endif /* NETPROC */
 
 	hold = 1;
 	if (options & F_SO_DEBUG)
@@ -547,19 +581,19 @@ main(argc, argv)
 
 #if INET6
         if (af == AF_INET6) {
-          struct icmpv6_filter filter;
+          struct icmp6_filter filter;
 	  int i;
 
-	  ICMPV6_FILTER_SETBLOCKALL(&filter);
+	  ICMP6_FILTER_SETBLOCKALL(&filter);
 
-	  ICMPV6_FILTER_SETPASS(ICMPV6_ECHO_REPLY, &filter);
-	  ICMPV6_FILTER_SETPASS(ICMPV6_DST_UNREACH, &filter);
-	  ICMPV6_FILTER_SETPASS(ICMPV6_PACKET_TOOBIG, &filter);
-	  ICMPV6_FILTER_SETPASS(ICMPV6_TIME_EXCEEDED, &filter);
-	  ICMPV6_FILTER_SETPASS(ICMPV6_PARAMETER_PROBLEM, &filter);
+	  ICMP6_FILTER_SETPASS(ICMP6_ECHO_REPLY, &filter);
+	  ICMP6_FILTER_SETPASS(ICMP6_DST_UNREACH, &filter);
+	  ICMP6_FILTER_SETPASS(ICMP6_PACKET_TOO_BIG, &filter);
+	  ICMP6_FILTER_SETPASS(ICMP6_TIME_EXCEEDED, &filter);
+	  ICMP6_FILTER_SETPASS(ICMP6_PARAM_PROB, &filter);
 
-          if (setsockopt(s, IPPROTO_ICMPV6, ICMPV6_FILTER, &filter, sizeof(struct icmpv6_filter)) < 0) {
-	    perror("setsockopt(IPPROTO_ICMPV6, ICMPV6_FILTER, ...)");
+          if (setsockopt(s, IPPROTO_ICMPV6, ICMP6_FILTER, &filter, sizeof(struct icmp6_filter)) < 0) {
+	    perror("setsockopt(IPPROTO_ICMPV6, ICMP6_FILTER, ...)");
 	    exit(1);
 	  };
 
@@ -579,7 +613,7 @@ main(argc, argv)
         {
 	  char host[NI_MAXHOST];
 
-	  if (getnameinfo((struct sockaddr *)&whereto, NRL_SA_LEN((struct sockaddr *)&whereto), host, sizeof(host), NULL, 0, NI_NUMERICHOST)) {
+	  if (getnameinfo((struct sockaddr *)&whereto, SA_LEN((struct sockaddr *)&whereto), host, sizeof(host), NULL, 0, NI_NUMERICHOST)) {
 	    fprintf(stderr, "ping: getnameinfo failed\n");
 	    exit(1);
 	  }
@@ -719,7 +753,7 @@ pinger()
 {
 	register struct icmp *icp;
 #if INET6
-	register struct icmpv6hdr *icmpv6 = (struct icmpv6hdr *)outpack;
+	register struct icmp6_hdr *icmp6 = (struct icmp6_hdr *)outpack;
 	struct msghdr msghdr;
 	struct iovec iovec;
 
@@ -739,7 +773,7 @@ pinger()
 
 	memset(&msghdr, 0, sizeof(struct msghdr));
 	msghdr.msg_name = (caddr_t)&whereto;
-	msghdr.msg_namelen = NRL_SA_LEN((struct sockaddr *)&whereto);
+	msghdr.msg_namelen = SA_LEN((struct sockaddr *)&whereto);
 	msghdr.msg_iov = &iovec;
 	msghdr.msg_iovlen = 1;
 	if (send_controllen) {
@@ -767,13 +801,13 @@ pinger()
 	    icp->icmp_cksum = in_cksum((u_short *)icp, iovec.iov_len);
 	    break;
 	  case AF_INET6:
-	    icmpv6->icmpv6_type = ICMPV6_ECHO_REQUEST;
-	    icmpv6->icmpv6_code = 0;
-	    icmpv6->icmpv6_cksum = 0;
-	    icmpv6->icmpv6_id = ident;
-	    icmpv6->icmpv6_seq = ntransmitted++;
-	    CLR(icmpv6->icmpv6_seq % mx_dup_ck);
-	    iovec.iov_len = datalen + sizeof(struct icmpv6hdr);
+	    icmp6->icmp6_type = ICMP6_ECHO_REQUEST;
+	    icmp6->icmp6_code = 0;
+	    icmp6->icmp6_cksum = 0;
+	    icmp6->icmp6_id = ident;
+	    icmp6->icmp6_seq = ntransmitted++;
+	    CLR(icmp6->icmp6_seq % mx_dup_ck);
+	    iovec.iov_len = datalen + sizeof(struct icmp6_hdr);
 	    break;
 	  default:
 	    fprintf(stderr, "ping: Don't know how to build an echo request for %s\n", nrl_afnumtoname(af));
@@ -827,7 +861,7 @@ pr_pack(buf, cc, from)
 {
 	register struct icmp *icp;
 #if INET6
-	register struct icmpv6hdr *icmpv6;
+	register struct icmp6_hdr *icmp6;
 #endif /* INET6 */
 	u_long l;
 	register int i, j;
@@ -860,9 +894,24 @@ pr_pack(buf, cc, from)
 	  case AF_INET6:
 	    if (msghdr) {
 	      struct cmsghdr *cmsghdr;
-	      for (cmsghdr = CMSG_FIRSTHDR(msghdr); cmsghdr; cmsghdr = CMSG_NXTHDR(msghdr, cmsghdr))
-		if ((cmsghdr->cmsg_level == IPPROTO_IPV6) && (cmsghdr->cmsg_type == IPV6_HOPLIMIT) && (cmsghdr->cmsg_len == sizeof(struct cmsghdr) + sizeof(int)) && (*(int *)CMSG_DATA(cmsghdr) >= 0) && (*(int *)CMSG_DATA(cmsghdr) < 256))
-		  sprintf(ttl, "%d", *(int *)CMSG_DATA(cmsghdr));
+	      for (cmsghdr = CMSG_FIRSTHDR(msghdr); cmsghdr; cmsghdr = CMSG_NXTHDR(msghdr, cmsghdr)) {
+#if 0
+		if ((cmsghdr->cmsg_level == IPPROTO_IPV6) && (cmsghdr->cmsg_type == IPV6_HOPLIMIT) && (cmsghdr->cmsg_len == CMSG_LEN(sizeof(int))) && (*(int *)CMSG_DATA(cmsghdr) >= 0) && (*(int *)CMSG_DATA(cmsghdr) < 256))
+#endif /* 0 */
+                if (cmsghdr->cmsg_level != IPPROTO_IPV6)
+                  continue;
+                if (cmsghdr->cmsg_type != IPV6_HOPLIMIT)
+                  continue;
+                if (cmsghdr->cmsg_len != CMSG_LEN(sizeof(int))) {
+		  printf("%d\n", CMSG_LEN(sizeof(int)));
+                  continue;
+		};
+                if ((*(int *)CMSG_DATA(cmsghdr) < 0) || (*(int *)CMSG_DATA(cmsghdr) > 255))
+                  continue;
+
+                sprintf(ttl, "%d", *(int *)CMSG_DATA(cmsghdr));
+                break;
+              };
 	    };
 	    hlen = 0;
             break;
@@ -872,7 +921,7 @@ pr_pack(buf, cc, from)
 	    break;
 	};
 
-        if (getnameinfo(from, NRL_SA_LEN(from), name, sizeof(name), NULL, 0, NI_NUMERICHOST)) {
+        if (getnameinfo(from, SA_LEN(from), name, sizeof(name), NULL, 0, NI_NUMERICHOST)) {
           fprintf(stderr, "ping: getnameinfo failed\n");
 	  exit(1);
 	};
@@ -897,10 +946,10 @@ pr_pack(buf, cc, from)
 #if INET6
 	switch(af) {
 	  case AF_INET6:
-	    icmpv6 = (struct icmpv6hdr *)buf;
-	    icmpdata = (char *)buf + sizeof(struct icmpv6hdr);
-	    icmpseq = icmpv6->icmpv6_seq;
-            if ((i = (icmpv6->icmpv6_type == ICMPV6_ECHO_REPLY)) && (icmpv6->icmpv6_id != ident))
+	    icmp6 = (struct icmp6_hdr *)buf;
+	    icmpdata = (char *)buf + sizeof(struct icmp6_hdr);
+	    icmpseq = icmp6->icmp6_seq;
+            if ((i = (icmp6->icmp6_type == ICMP6_ECHO_REPLY)) && (icmp6->icmp6_id != ident))
               return;
 	    break;
 	  case AF_INET:
@@ -1014,7 +1063,7 @@ pr_pack(buf, cc, from)
 		if (af == AF_INET)
 		  pr_icmph(icp);
 		else
-		  pr_icmpv6h(buf);
+		  pr_icmp6h(buf);
 #else /* INET6 */
 		    pr_addr(from->sin_addr.s_addr));
 		pr_icmph(icp);
@@ -1356,74 +1405,74 @@ pr_icmph(icp)
 
 #if INET6
 /*
- * pr_icmpv6h --
+ * pr_icmp6h --
  *	Print a descriptive string about an ICMPv6 header.
  */
-pr_icmpv6h(char *cp)
+pr_icmp6h(char *cp)
 {
-  struct icmpv6hdr *icmpv6 = (struct icmpv6hdr *)cp;
+  struct icmp6_hdr *icmp6 = (struct icmp6_hdr *)cp;
 
-  switch(icmpv6->icmpv6_type) {
-    case ICMPV6_ECHO_REPLY:
+  switch(icmp6->icmp6_type) {
+    case ICMP6_ECHO_REPLY:
       printf("Echo Reply\n");
       break;
-    case ICMPV6_DST_UNREACH:
-      switch(icmpv6->icmpv6_code) {
-        case ICMPV6_UNREACH_NOROUTE:
+    case ICMP6_DST_UNREACH:
+      switch(icmp6->icmp6_code) {
+        case ICMP6_DST_UNREACH_NOROUTE:
 	  printf("Destination Route Not Available\n");
 	  break;
-        case ICMPV6_UNREACH_ADMIN:
+        case ICMP6_DST_UNREACH_ADMIN:
 	  printf("Destination Administratively Prohibited\n");
 	  break;
-        case ICMPV6_UNREACH_NOTNEIGHBOR:
+        case ICMP6_DST_UNREACH_NOTNEIGHBOR:
 	  printf("Destination Neighbor Unreachable\n");
 	  break;
-        case ICMPV6_UNREACH_ADDRESS:
+        case ICMP6_DST_UNREACH_ADDR:
 	  printf("Destination Address Unreachable\n");
 	  break;
-        case ICMPV6_UNREACH_PORT:
+        case ICMP6_DST_UNREACH_NOPORT:
 	  printf("Destination Port Unreachable\n");
 	  break;
         default:
-	  printf("Dest Unreachable, Bad Code: %d\n", icmpv6->icmpv6_code);
+	  printf("Dest Unreachable, Bad Code: %d\n", icmp6->icmp6_code);
 	  break;
       }
-      pr_retip(cp + sizeof(struct icmpv6hdr));
+      pr_retip(cp + sizeof(struct icmp6_hdr));
       break;
-    case ICMPV6_PACKET_TOOBIG:
+    case ICMP6_PACKET_TOO_BIG:
       printf("Packet too big\n");
-      pr_retip(cp + sizeof(struct icmpv6hdr));
+      pr_retip(cp + sizeof(struct icmp6_hdr));
       break;
-    case ICMPV6_TIME_EXCEEDED:
-      switch(icmpv6->icmpv6_code) {
-        case ICMPV6_EXCEEDED_HOPS:
+    case ICMP6_TIME_EXCEEDED:
+      switch(icmp6->icmp6_code) {
+        case ICMP6_TIME_EXCEED_TRANSIT:
 	  printf("Hop limit exceeded\n");
 	  break;
-        case ICMPV6_EXCEEDED_REASSEMBLY:
+        case ICMP6_TIME_EXCEED_REASSEMBLY:
 	  printf("Frag reassembly time exceeded\n");
 	  break;
         default:
-	  printf("Time exceeded, Bad Code: %d\n", icmpv6->icmpv6_code);
+	  printf("Time exceeded, Bad Code: %d\n", icmp6->icmp6_code);
 	  break;
       }
-      pr_retip(cp + sizeof(struct icmpv6hdr));
+      pr_retip(cp + sizeof(struct icmp6_hdr));
       break;
-    case ICMPV6_PARAMETER_PROBLEM:
-      switch(icmpv6->icmpv6_code) {
-        case ICMPV6_PARAMPROB_HDR:
+    case ICMP6_PARAM_PROB:
+      switch(icmp6->icmp6_code) {
+        case ICMP6_PARAMPROB_HEADER:
 	  printf("Parameter Problem\n");
 	  break;
-        case ICMPV6_PARAMPROB_NEXTHDR:
+        case ICMP6_PARAMPROB_NEXTHEADER:
 	  printf("Bad Next Header\n");
 	  break;
-        case ICMPV6_PARAMPROB_OPTION:
+        case ICMP6_PARAMPROB_OPTION:
 	  printf("Unrecognized IPv6 Option\n");
 	  break;
       }
-      pr_retip(cp + sizeof(struct icmpv6hdr));
+      pr_retip(cp + sizeof(struct icmp6_hdr));
       break;
     default:
-      printf("Bad ICMPv6 type: %d\n", icmpv6->icmpv6_type);
+      printf("Bad ICMPv6 type: %d\n", icmp6->icmp6_type);
   }
 }
 #endif /* INET6 */
@@ -1457,9 +1506,6 @@ pr_iph(ip)
 	    ip->ip_v, ip->ip_hl, ip->ip_tos, ip->ip_len, ip->ip_id);
 	(void)printf("   %1x %04x", ((ip->ip_off) & 0xe000) >> 13,
 	    (ip->ip_off) & 0x1fff);
-#if defined(linux) && !defined(__GLIBC__)
-#define ip_sum ip_csum
-#endif /* defined(linux) && !defined(__GLIBC__) */
 	(void)printf("  %02x  %02x %04x", ip->ip_ttl, ip->ip_p, ip->ip_sum);
 	(void)printf(" %s ", inet_ntoa(*(struct in_addr *)&ip->ip_src.s_addr));
 	(void)printf(" %s ", inet_ntoa(*(struct in_addr *)&ip->ip_dst.s_addr));
@@ -1472,7 +1518,7 @@ pr_iph(ip)
         } else
 	  if (af == AF_INET6) {
 	/* IPv6 header (no options for now) */
-	  struct ipv6hdr *ipv6 = (struct ipv6hdr *)bufp;
+	  struct ip6_hdr *ip6 = (struct ip6_hdr *)bufp;
           struct sockaddr_in6 sin6;
           char hbuf1[64], hbuf2[64];
 
@@ -1482,13 +1528,13 @@ pr_iph(ip)
 #endif /* SALEN */
           sin6.sin6_family = AF_INET6;
 
-          sin6.sin6_addr = ipv6->ipv6_src;
+          sin6.sin6_addr = ip6->ip6_src;
           if (getnameinfo((struct sockaddr *)&sin6, sizeof(struct sockaddr_in6), hbuf1, sizeof(hbuf1), NULL, 0, (options & F_NUMERIC) ? NI_NUMERICHOST : 0)) {
             fprintf(stderr, "ping: getnameinfo failed\n");
             exit(1);
           };
 
-          sin6.sin6_addr = ipv6->ipv6_dst;
+          sin6.sin6_addr = ip6->ip6_dst;
           if (getnameinfo((struct sockaddr *)&sin6, sizeof(struct sockaddr_in6), hbuf2, sizeof(hbuf2), NULL, 0, (options & F_NUMERIC) ? NI_NUMERICHOST : 0)) {
             fprintf(stderr, "ping: getnameinfo failed\n");
             exit(1);
@@ -1496,9 +1542,9 @@ pr_iph(ip)
 
 	  (void)printf("V P FlowID Len  Nxt Hop Src -> Dst\n");
 	  (void)printf("%1x %1x %06x %04x %2x  %2x  %s -> %s\n",
-                ipv6->ipv6_version, ipv6->ipv6_priority,
-                ntohl(ipv6->ipv6_flowid), ntohs(ipv6->ipv6_len),
-                ipv6->ipv6_nextheader, ipv6->ipv6_hoplimit, hbuf1, hbuf2);
+                (ip6->ip6_vfc >> 4), (ip6->ip6_vfc & 0x0f),
+                ntohl(ip6->ip6_flow), ntohs(ip6->ip6_plen),
+                ip6->ip6_nxt, ip6->ip6_hlim, hbuf1, hbuf2);
 	}
 #endif /* INET6 */
 }
@@ -1548,8 +1594,8 @@ pr_retip(ip)
 	  int size;
 	 
 	  /* Parse packet for transport layer data */
-	  proto = ((struct ipv6hdr *)cp)->ipv6_nextheader;
-	  size = sizeof(struct ipv6hdr);
+	  proto = ((struct ip6_hdr *)cp)->ip6_nxt;
+	  size = sizeof(struct ip6_hdr);
 
 	  do {
 	    cp += size;
@@ -1557,26 +1603,26 @@ pr_retip(ip)
 #if 0
 		case IPPROTO_HOP:
 		  /* punt for now*/
-		  size = ((struct ipv6_opthdr *)cp)->oh_nexthdr + 8;
-		  proto = ((struct ipv6_opthdr *)cp)->oh_nexthdr;
+		  size = ((struct ip6_opthdr *)cp)->oh_nexthdr + 8;
+		  proto = ((struct ip6_opthdr *)cp)->oh_nexthdr;
 		  break;
 		case IPPROTO_AH:
 		  size = ((struct ipsec_ah *)cp)->ah_datalen * 8 + 8;
 		  proto = ((struct ipsec_ah *)cp)->ah_nexthdr;
 		  break;
 		case IPPROTO_ROUTING:
-		  if (((struct ipv6_srcroute0 *)cp)->i6sr_type == 0) {
-		    size = ((struct ipv6_srcroute0 *)cp)->i6sr_numaddrs *
+		  if (((struct ip6_srcroute0 *)cp)->i6sr_type == 0) {
+		    size = ((struct ip6_srcroute0 *)cp)->i6sr_numaddrs *
 		      sizeof(struct in6_addr) + 8;
-		    proto = ((struct ipv6_srcroute0 *)cp)->i6sr_nexthdr;
+		    proto = ((struct ip6_srcroute0 *)cp)->i6sr_nexthdr;
 		  } else {
-		    (void)fprintf(stderr, "ping: ipv6 routing type unknown\n");
+		    (void)fprintf(stderr, "ping: ip6 routing type unknown\n");
 		    return;
 		  }
 		  break;	      
 		case IPPROTO_FRAGMENT:
 		  size = 8;
-		  proto = ((struct ipv6_fraghdr *)cp)->frag_nexthdr;
+		  proto = ((struct ip6_fraghdr *)cp)->frag_nexthdr;
 		  break;
 #endif /* 0 */
 		default:
@@ -1647,9 +1693,9 @@ usage()
 #if INET6
 	    "[-a family] "
 #endif /* INET6 */
-#if NETSEC
-	    "[-S security request] "
-#endif /* NETSEC */
+#if NETPROC
+	    "[-S request] "
+#endif /* NETPROC */
 	    "[-c count] [-i wait] [-l preload] [-p pattern] [-s packetsize] host\n");
 	exit(1);
 }
