@@ -128,9 +128,6 @@ static char sccsid[] = "@(#)main.c	8.1 (Berkeley) 6/6/93";
 #include <sys/file.h>
 
 #include <netinet/in.h>
-#if INET6
-#include <netinet6/in6.h>
-#endif /* INET6 */
 
 #include <arpa/inet.h>
 
@@ -149,8 +146,6 @@ static char sccsid[] = "@(#)main.c	8.1 (Berkeley) 6/6/93";
 #define	TIMEOUT		5		/* secs between rexmt's */
 
 #if INET6
-#include "support.h"
-
 int     fv4;
 int     fv6;
 struct  sockaddr_in6 peeraddr6;
